* Graphics for meta analysis - Figure 1.1
* 
*
* Jeff Tomlinson, January 2012
;

* Initial setup.;
options
    dsoptions="note2err"
    msglevel=I details fullstimer
    ls=150 nocenter missing=' '
    obs=max
    nomprint nosymbolgen nomlogic
    missing=.
;
title;
footnote;

* Clear work area. ;
proc datasets nolist lib=work kill;
run;

* Get the Clopidogrel data directly from the spreadsheet. 
* First define a LIBNAME for the Excel spreadsheet. ;
libname xl excel './excel/clopidogrel.xls' access=readonly;
%let dsin=xl.'GenStat Data$'n;  * Input dataset name. (Sheet of the Excel workbook).;

data studyci(keep=study rr logrr k loglo loghi wt linen label)
     split  (keep=study linen label2)
     fixeff (keep=study linen sumwt sumwtsq sumwtlog com secom hicom locom k label2);
  set &dsin. end=atend;
  length label label2 $30;
  retain k 1 sumwt sumwtsq sumwtlog maxwt 0 ;

  if missing(rr) then do;
    logrr = 0;
    loglo = 0;
    loghi = 0;
    wt    = 0;
  end;
  else do;
    logrr = log10(rr);
    loglo = log10(lo95);
    loghi = log10(hi95);
    wt    = 1 / ((loghi-logrr)/1.96)**2;
    maxwt = max(maxwt, wt);
  end;

  sumwt    + wt;
  sumwtsq  + wt**2;
  sumwtlog + (logrr*wt);
  linen = _n_;
  label = tranwrd(tranwrd(cats(r1, '/', n1, 'v', r2, '/', n2),'.','NR'),'v', ' vs ');
  output studyci;

  if atend then do;
    study = '-';
    linen = _N_+1;
    label2 = ' ';
    output split;

    call symputx('ntrial', _n_);
    call symputx('sqmaxwt', sqrt(maxwt));

    com = sumwtlog/sumwt;
    secom = sqrt(1/sumwt);
    hicom = com+1.96*secom;
    locom = com-1.96*secom; 
    study = 'Fixed effects';
    linen = _N_+2;
    label2 = compbl(cat(put(10**com,8.2)
           , ' (',put(10**locom,8.2-L)
           ,', ',put(10**hicom,8.2-L),')'));
    output fixeff;
  end;
run;

* Calculate Tau^2 for the random effect calculation.;
data tau2(keep=k df Q I2 C tau2);
  merge fixeff studyci end=atend;
  by k;
  Q + wt*((logrr-com)**2);
  if atend then do;
    df = &ntrial.-1;
    I2 = 100*(Q-df)/Q;
    C = sumwt - sumwtsq/sumwt;
    if Q gt df then tau2 = (Q-df)/C;
               else tau2 = 0;
    output;
  end;
run;

* Calculate the random effects. ;
data raneff(keep=study linen sumwt sumwtlog com secom hicom locom k linen label2);
  merge tau2 studyci end=atend;
  by k;
  length label2 $30;
  wt2 = 1 / (((loghi-logrr)/1.96)**2 + tau2);
  sumwtlog + wt2*logrr;
  sumwt + wt2;
  if atend then do;
    com    = sumwtlog/sumwt;
    secom  = sqrt(1/sumwt);
    hicom  = com+1.96*secom;
    locom  = com-1.96*secom;  
    study  = 'Random effects';
    linen  = &ntrial.+3;
    label2 = compbl(cat(put(10**com,8.2)
           , ' (',put(10**locom,8.2-L),', '
           , put(10**hicom,8.2-L),')') );
    output raneff;
  end;
run;

* Combine the study CI information with the fixed effects and random effects data. ;
data plotbase;
  set studyci(in=indata)
      split(in=insplit)
      fixeff(in=infix)
      raneff(in=inran)
  ;
  by linen;
  ord+1;
run;


* Use a modified box plot to plot the boxes and the 95% CI.
* The body of the box will the square prportional to the weight.  
* The whiskers will be the 95% CI. 
;

* Some calculation is required to ensure that the WEIGHT blocks are square.
* The limit of the size of the blocks is down to the maximum BOXWIDTH size.
* BOXWIDTH can have values of 0..1, but a value less than 1 gives are better
* look to the plot.  We will need to know the size of the plot area (in pixels).
* From this we can calculate the height of the largest block = Number of 
* pixels per line x 0.6 = 0.6 x 2000/(26+3) = 41.  
*
* Pixels per unit for the X axis is 1500/(0.8 - (-0.8)) = 937.5, given the 
* maximum length will be 41 pixels, the scaling factor will be 41/(2 x 937.5) = 0.022
;
%let maxrr=0.8;     * Max RR for the X axis. ;
%let plwd=1500;     * Width of plot area in pixels. ;
%let plht=2000;     * Height of plot area in pixels. ;
%let hfac=0.7;      * Proportion of max box width for the box. ;

%let WEIGHT=%sysevalf((&plht./&plwd.) * (&maxrr./(&ntrial.+3) * &HFAC.) );
%put &weight.;

%let offtheplot=-20; * Far outside the plot area.  ;

data forbox (keep=ord study stat value linen);
  set plotbase(drop=ord);
  
  length stat $12;

  if wt le 0 then do;
    wt  = 0;
    logrr = &offtheplot.;
    loglo = &offtheplot.;
    loghi = &offtheplot.;
  end;
  
  ord+1;
  stat = 'MIN';
  value = loglo;
  output;
  
  ord+1;
  stat = 'MAX';
  value = loghi;
  output;
  
  if not missing(wt) then do;
    _wt = sqrt(wt)/&sqmaxwt.;
    weight = &WEIGHT.*_wt;
    ord+1;
    stat = 'Q1';
    value = logrr-weight;
    output;
    
    ord+1;
    stat = 'Q3';
    value = logrr+weight;
    output;
    
    ord+1;
    stat = 'BOXWIDTH';
    value = &hfac.*_wt;
    output;
  end;
run;

* Determine where the labels for the overall diamons are to be 
* placed.  To the right of the largest 95 % limit. ;
proc sql noprint;
  select max(hicom) into :maxhicom from plotbase;
quit;
%put &maxhicom;

* Construct the data for the overall diamond.  
* XC, YC are the coordinates of the labels.
* WX, YVLO, YVHI are the limits for the diamond.  
;
data forover(keep=ord xv linen2 yvlo yvhi xc yc label2);
   set plotbase(keep=study sumwt com locom hicom linen study label2 );
   where (study ? 'ffects');

  linen2 = linen;

  xc = &maxhicom.;
  yc = linen-0.25;

  xv = locom;
  yvlo = linen;
  yvhi = linen;
  ord +1;
  output;

  xc = .;
  yc = .;
  xv = com;
  yvlo = linen+0.4;
  yvhi = linen-0.4;
  ord +1;
  output;

  xv = hicom;
  yvlo = linen;
  yvhi = linen;
  ord +1; 
  output;
run;

* Y2 axis tick values will be formatted with the risk values.  
* Construct a format to do this. 
* This is needed because the Y2 axis is a linear axis unlike the Y axis
* which is a discrete axis.  Both axes have the same number of ticks and
* are ordered the same, so they will be in sequence.  A linear axis is 
* needed in order to use SCATTERPLOT and BANDPLOT to the overall diamonds
* and their labels.
;
data _fstud;
  length label $30;
  set plotbase(rename=(linen=start)) end=atend;
  where label is not null;
  retain fmtname 'frisk' type 'n';
  output;
  if atend then do;
    hlo = 'O';
    label = ' ';
    output;
  end;
run;
proc format library=work cntlin=_fstud;
run;


* All of the plot data into one dataset. ;
data plot1;
   merge forbox
         forover
         plotbase(rename=(study=study2 linen=linen3) drop=label2)
   ;
   by ord;
run;

* Define the template for the plot. 
* We will use a linear X axis to plot the log of the RR and intervals on.
* The linear axis has more scope for customization than the log axis. 
* The X axis labelling will be done using footnotes.
;
proc template;
  define statgraph fig1_temp;
  
    begingraph /  designwidth  = 1200px
                  designheight = 1800px;

      * Titles and footnotes. ;
      dynamic _title1 _title2 _foot1;

      * Headers of the Y and Y2 axes. ;
      dynamic _hdstudy _hdrisk;

      * Number of lines (# studies + 3) on the plot. ; 
      dynamic _maxyax;

      * X axis labels ;
      dynamic _highalab _highblab _xaxlab;

      * _TICKAT are the values for the ticks on the X axis. 
      * _TICKLAB are the labels for the ticks.
      * _MINXAX ans _MAXXAX are the limits for the X axis. ;
      dynamic _tickat _ticklab _minxax _maxxax;
 
      * Allow for 2 title lines. ;
      EntryTitle halign = left _title1;
      EntryTitle halign = left _title2;
      
      * Axes headers. ;
      EntryTitle halign=left _hdstudy  
                 halign=right _hdrisk /pad=(left=200 right=100)  ;
      
      * Plot area ;
      layout overlay /  
        walldisplay = (fill outline)
        yaxisopts   = (
            display    = (ticks tickvalues line)
            reverse    = true
            linearopts = (integer=true)
            griddisplay=on
                        )
        y2axisopts   = (
            display    = (ticks tickvalues line)
            reverse    = true
            linearopts = (integer=true  
                          viewmin=1 viewmax=_maxyax
                          tickvaluesequence=(start=1 end=_maxyax increment=1)  
                          tickvalueformat=frisk.
                          )
                        )
        xaxisopts   = ( type       = linear
            display    = (ticks tickvalues line)
            offsetmin  = 0
            offsetmax  = 0
            linearopts = (tickvaluelist =  _tickat
                          tickdisplaylist= _ticklab
                          viewmin=_minxax viewmax=_maxxax
                          )
                        );

        * Vertical reference line. ;
        referenceline x=0.0 / lineattrs = (pattern=shortdash  color=green);
        
        * Horizontal reference line dividing the study area from the effects. ;
        referenceline y= %eval(&ntrial.+1) / 
                                lineattrs = (pattern=solid color=green) 
                                yaxis=y2 ;

        * The "OVERALL" red diamond. ;
        bandplot x=xv limitupper=yvhi limitlower=yvlo / 
              display=(fill) 
              yaxis=y2 
              fillattrs=(color=red)
        ;
        scatterplot x=xc y=yc / 
              markerattrs=(size=0) 
              datalabel=label2 
              yaxis=y2
              datalabelattrs=(color=red)
        ;
 
        * Study box and lines. ;
        boxplotparm x=study y=value stat=stat / 
              orient=horizontal
              capshape=serif
              fillattrs=(color=black)
              whiskerattrs=(thickness=2px) 
        ;
      
      endlayout;

      * Define the X axis labels. ;
      EntryFootnote halign=left  _highalab
                    halign=right _highblab
                    /pad=(left=350 right=350)  ;
      EntryFootnote halign=center _xaxlab
                    /textattrs = (weight=bold) ;

      * Allow one (optional) footnote. ;
      EntryFootnote _foot1;
      
    endgraph;
  end;
run;

* Template for the fonts and colours to be used in the plot. ; 
proc template;
   edit styles.Statistical as locstyle;

   style GraphFonts /
      'GraphTitleFont'    = ("<sans-serif>, <MTsans-serif>",14pt,bold)
      'GraphFootnoteFont' = ("<sans-serif>, <MTsans-serif>",14pt)
      'GraphDataFont'     = ("<sans-serif>, <MTsans-serif>",14pt)
      'GraphUnicodeFont'  = ("<MTsans-serif-unicode>",9pt)
      'GraphValueFont'    = ("<sans-serif>, <MTsans-serif>",14pt)
      'GraphLabelFont'    = ("<sans-serif>, <MTsans-serif>",14pt)
      'GraphAnnoFont'     = ("<MTmonospace>, <monospace>, Courier, Cumberland AMT",8pt)
                ;
   class colors /
      'docbg' = white;
   end;
run;

* Define the style to be used and the image resolution. ;
ods listing style=locstyle image_dpi=300 gpath="./" ;

* Define the image type and dimensions. ;
ods graphics /  reset 
                imagename="Fig1_1"
                imagefmt = png 
                width    = 8in
                height   = 8in
;
              
* Render the plot. ;
proc sgrender data = plot1
           template = fig1_temp;
   dynamic 
    _title1   = " "
    _hdstudy  = "Study"
    _hdrisk   = "Risk (B vs A)"
    _maxyax   = %eval(&ntrial.+3)
    _highalab ='Higher risk for A: *1 or *17'
    _highblab = 'Higher risk for B: *2-*8'
    _xaxlab   = 'Ratio and 95% CI'
    _minxax   = -&maxrr.
    _maxxax   = &maxrr.
    _tickat   ="-0.70 -0.52 -0.40 -0.30 -0.22 -0.15 -0.10 -0.05 0.00 0.30 0.48 0.60 0.70"
    _ticklab  ="'0.2' ' '   ' '   '0.5' ' '   ' '   ' '   ' '   '1'  '2'  ' '  ' '  '5'"
;
 run;
