* Graphics for meta analysis - Figure 1.2
* Sub-group analysis of SSRI data.
*
* Jeff Tomlinson, January 2012
;

*======================================================================;
* Initial setup.
*======================================================================;
options
    dsoptions="note2err"
    mergenoby=warn dkricond=warn dkrocond=warn
    compress=NO
    msglevel=I details fullstimer
    ls=150 nocenter missing=' '
    obs=max
    nomprint nosymbolgen nomlogic
;
title;
footnote;

* Clear work area. ;
proc datasets nolist lib=work kill;
run;

*======================================================================;
* Define the macro variable that will control the output generated.
*======================================================================;

* Name the output file. and provide titles and footnotes. ;
%let opfile=Fig1_2.png;
%let title=; 
%let footnote=; 
%let trtname=SSRI;
%let cmpname=Placebo;

* Layout macro variables. ;
%let pltper=80;			* Percentage of plot for text columns ;
%let stcolw=16;			* Percentage for STUDY column. ;
%let dtcolw=7;			* Percentage for DATA columns. ;

* Fonts used. ;
%let tfont=Albany AMT;		* Normal text. ;
%let bfont=&tfont./bold;	* Headings, sub-headings. ;
%let ifont=&tfont./italic;* Additional stats. ;

* Text and annotation size control ;
%let diaht=0.8;  * Proportion of the line height for the diamond. ;
%let textht=1.1; * Size of TEXT. ;
%let ticksz=0.2; * The size pf the ticks at the end of the CI lines. ;
%let boxscale=8; * Scale for the boxes. ;

* Page size data. ;
%let ppline=50;			* Indication of the # of pixels per line. ;
%let addlines=5;		* Additional lines above those for the Y axis. ;
%let xwid=7.5;			* Width of plot in inches. ;
%let ywid=10.5;			* Height of plot in inches. ;

*======================================================================;
* Get the data into a format that can be used to calculate statistics.
* From the SSRI data we will need SDs for the individual treatment arms
* and the overall variance for a study.  We will also need the full
* drug name, as only the initial letters are in SSRI.
*======================================================================;

* From DRUG code to full name. ;
proc format;
  value $drugfl
    'F' = '1.2.1 Fluoxetine'
    'V' = '1.2.4 Venlafaxine'
    'N' = '1.2.2 Nefazodone'
    'P' = '1.2.3 Paroxetine'
  ;
run;

* Define the names of the fields required for the output, based on the 
* SSRI data. ;
libname xla excel './ssri.xls' access=readonly;

data ssri(keep=drug study n1 change1 sd1 n2 change2 sd2 ES var wx);
  attrib
    drug    length=$80 label='Drug (for sub-group)'
    study   length=$80 label='Study or Subgroup'
    sd1     label='SD'
    sd2     label='SD'
		change1 label='Mean'
		change2 label='Mean'
		n1      label='Total'
		n2      label='Total'
    Var     label='Variance'
    ES      label='Difference'
  ;
  set xla.'GenStat Data$'n end=atend;
  format drug study;
  drug = put(drug, drugfl.);

  sd1 = change1/d1;
  sd2 = change2/d2;
  Var = sd1**2/n1 + sd2**2/n2;
  ES  = change1-change2;
run;
libname xla clear;

* Order by drug name and variance. ;
proc sort data=ssri;
  by drug descending var;
run;

*======================================================================;
* Add the initial weight for each study (1/variance) and calculate the
* 95% CI for the difference in changes from baseline.
*======================================================================;
data ford1;
  set ssri;
  by drug descending var;
  attrib
    w       label='Weight '
    lowerci label='Lower 95% CI'
    upperci label='Upper 95% CI'
  ;
	if first.drug then do;
		drugord+1;
	end;

  w = 1/var;
  lowerci = ES+probit(0.025)*sqrt(var);
  upperci = ES+probit(0.975)*sqrt(var);
run;

title 'By Study ';
proc print data=ford1;
  var drug study n1 change1 sd1 n2 change2 sd2 ES lowerci upperci;
run;
title;

*======================================================================;
* Macro CalcRandEff 
* Calculate the random effects statistics etc for the sub-groups and overall.
*
* INDS    - Input dataset.
* OUTDS   - Output dataset.
* BY      - By variable for subsets, if blank, then do not subset
* KEEP    - List of additional variables to keep from INDS
* EFFECT  - Effect variable in INDS
* NUMT    - Variable in INDS with number of subjects on treatment
* NUMC    - Variable in INDS with number of subjects on comparator in study.
* VAR     - Variance.
*======================================================================;
%macro CalcRandEff(inds=, outds=, by=, keep=, effect=, numt=, numc=, var=);
  * If no BY is specified then calculate the statistics on the total ;
  %if &by. eq %then %let by=_by_;

  * Construct a local dataset with an fixed BY value. ;
  data _cix(keep=&by. &keep. &effect. &numt. &numc. &var. );
    set &inds.;
    _by_ = 1;
  run;
* Q is CHI2  DF = N-1;
  data &outds.(keep=&by. &keep. sumn1 sumn2 Q Df TauSq Isquared es var se lowerci upperci
                     Z pchi pZ w)
		;
    attrib
      w       label='Weight'
      var     label='Variance'
    ;
    * Fixed Effects. ;
    sumw     = 0;
    sumesw   = 0;
    sumessqw = 0;
    sumwsq   = 0;
    sumn1    = 0;
    sumn2    = 0;
    Df       = 0;
    put ;
    do until(last.&by.);
      set _cix;
      by &by.;
      w        = 1/&var.;
      sumw     = sumw+w;
      sumesw   = sumesw+&effect.*w;
      sumessqw = sumessqw+&effect.**2*w;
      sumwsq   = sumwsq+w**2;
      sumn1    = sumn1+&numt.;
      sumn2    = sumn2+&numc.;
      Df       = Df+1;
    end;
    Q        = sumessqw-sumesw**2/sumw;
    Df       = Df-1;
    C        = sumw - sumwsq/sumw;
    TauSq    = (Q-Df)/C;
    Isquared = 100*(Q-Df)/Q;
		_w = sumw;
    * Random Effects. ;
    sumw   = 0;
    sumesw = 0;
    do until(last.&by.);
      set _cix;
      by &by.;
      vartot = &var. + TauSq;
      w      = 1/vartot;
      sumw   = sumw+w;
      sumesw = sumesw+&effect.*w;
    end;  

    ES      = sumesw/sumw;
    Var     = 1/sumw;
    SE      = sqrt(Var);
    lowerci = ES+probit(0.025)*SE;
    upperci = ES+probit(0.975)*SE;
		Z = ES/SE;
		w = _w;
		pchi = 1-probchi(Q, df);
		pZ = 2*(1-probnorm(Z));
    %if &by. eq _by_ %then %do;
			  drug = 'Overall';
				drugord = 999;
		%end;
		output &outds.;
  run;

  title "Dataset: &outds. ";
  proc print;
  run;
  title;
%mend;

*======================================================================;
* Calculate the statistics for the sub-groups and overall
*======================================================================;

* Stats etc by sub-group (DRUG) ;
%CalcRandEff(inds=ford1, outds=fsubg, by=drugord, keep=drug STUDY, effect=ES, numt=n1, numc=n2, var=var);

* Stats Overall ;
%CalcRandEff(inds=ford1, outds=ftot, keep=drug drugord, effect=ES, numt=n1, numc=n2, var=var);

* Calculate the percentage weights ;
data ford2;
	totwx = 0;
	maxw = 0;
	do until(atend);
		set ford1 end=atend;
		_w = sqrt(w);
		totwx = totwx+_w;
	end;
	atend = 0;
	do until(atend);
		set ford1 end=atend;
		pwx = round(sqrt(w)/totwx, 0.001);
		maxw = max(maxw, pwx);
		output;
	end;
	call symputx('maxw', maxw);
	call symputx('totw', totwx);

run;
%put Max W = &maxw.;

* Add the study data, sub-group (drug) and overall to one Dataset
* ordered by DRUG, with the sub-group information after all of the study data
* and the overall information at the end ;
data allt1;
  set ford2(in=indat)
      fsubg(in=insubg rename=(sumn1=n1 sumn2=n2))
      ftot (in=intot  rename=(sumn1=n1 sumn2=n2))
  ;
  by drugord;
  attrib
    studrec   label='Study record'
    subgrec   label='Sub-group record'
    totrec    label='Total/overall record'
  ;
	if first.drugord then _totpwx = 0;
  studrec = indat;
  subgrec = insubg;
  totrec = intot;
	if indat then _totpwx+pwx;
	else if insubg then pwx = round(_totpwx, 0.001);
	else if intot then pwx = 1;
run;
title 'Individual CIs, group and overall statistics ';
proc print data=allt1;
  var drug study n1 change1 sd1 n2 change2 sd2 ES lowerci upperci Isquared TauSq pwx;
run;
title;

*======================================================================;
* Construct the data to display.  
* The subgroup and overall records will be expanded to have additional 
* lines for axes and other information
*======================================================================;

* Expand the base dataset. With one record for each non-blank record for the 
* output.  The type of the record are identified by the RECTYPE variable.
* RECTYPE determine the format of the output line.
* DAT is the basic dataline, with the 95% CI plot for the study.
* SBG is the subgroup line - with the diamond as the graphic.
;

* Format for P Values. ;
proc format;
	value px 
		0       -< 0.0001  = [pvalue8.5]
		0.0001  -< 0.001   = [pvalue8.4]
		other              = [pvalue8.3]
		;
run;

* Add the summary lines for each drug. ;
data addsmry;
  set allt1(keep=drug drugord study n1 change1 sd1 n2 change2 sd2 es 
            lowerci upperci subgrec totrec studrec w Isquared TauSq
            df Q Z pZ pchi pwx) end=atend;
  by drug;
  
  attrib
    sd1  format=d7.
    sd2  format=d7.
    w    label='Weight'
		pwx	 label='Weight'
    rectype length=$3 label='Record type (DAT=Data, SBG=summary line)'
		sgline1-sgline3 length=$200 label='Sub-group summary lines' 
	;
  retain totlines 0;

  ord+1;

  if subgrec or totrec then do;
    if totrec then  call symputx('oves', ES);

		if totrec then study = 'Total (95% CI)';
		          else study = 'Subtotal (95% CI)';
    
    rectype ='SBG';
    
		sgline1 = 'Heterogeneity: Tau'|| 'b2'x ||'='||trim(put(TauSq,7.2-l))||';'
					||'Chi'|| 'b2'x ||'='||trim(put(Q,7.2-l))
          ||', df='||trim(put(df,5.-l))
          ||' (P='||compress(put(pchi,px.))||');'
					||' I'|| 'b2'x ||'='||trim(put(Isquared,4.1-L))||'%';
    
		sgline2 = 'Test for overall effect: Z='||trim(put(Z,8.2-L))
          ||' (P='||compress(put(pZ,px.))||'); ';
		sgline3 = ' ';
		nlines = &addlines.;  * Blank/drug at start.  sub total/hetro/Test for overal at end. ;
	  output;
  
  end;
  
  else do;
    rectype = 'DAT';
		nlines = 1;
    output;
  end;

	totlines + nlines;
	if atend then do;
		call symputx('numlines', totlines);
	end;
run;  
%put Number of lines : &numlines.;


data plotbase;
  set addsmry(drop=ord) end=atend;
	by drugord;
  attrib
		col1-col8 length=$24
		lab1-lab8 length=$24
  ;
	array cols[*] col1-col8;
	array labs[*] lab1-lab8;
	array vars[7] change1 sd1   n1  change2 sd2   n2   pwx ;
	array fmts[7] $20 _temporary_  (
								'8.2'   '8.3' '8' '8.2'   '8.3' '8'  'percent8.1'
  );
	retain ord 0;

	if not first.drugord then ord = ord-1;
	else if ord eq 0     then ord = -2;
	                     else ord = ord-&addlines.;

	do i=1 to dim(vars);
		cols[i] = putn(vars[i], fmts[i]);
		labs[i] = vlabel(vars[i]);
	end;

  if n(ES, lowerci, upperci) eq 3 then do;
    cols[8] = compbl(put(es,8.2)||' ['||put(lowerci,5.2)||','||put(upperci,5.2)||']');
		labs[8] = 'Mean Diff + 95% CI';
    if n(pwx) then wx = pwx/&maxw.;
  end;
	
	if atend then do;
		call symputx('ordx', -ord);
	end;
run;
%put NUMLINES: &numlines. ORDX: &ordx.;;

* One line for each study. ;
data plotdata;
	set plotbase;
	where rectype='DAT';
	grp = ord;
	lval = lowerci;
	output;
	lval = upperci;
	output;
	lval = .;
	output;
run;

* How many pixels for X and Y ;
%let ypix=%sysevalf(&ppline.*(&numlines.+&addlines.));
%let xpix=%sysevalf(300*&xwid);
%put XPIX=&xpix. YPIX=&ypix.;

* Declare the annotate macros. ;
%annomac;

* Define the WEIGHT boxes. ;
data box(drop=es ord wx rectype _relx _rely);
	set plotbase(keep=es ord wx rectype lowerci upperci);
	where rectype eq 'DAT';
	%dclanno;
	when = 'A';
	if n(wx) then do;
		%system(2,2,2);		* Data values. ;
		%move(es, ord);
		_rely = &boxscale.*1.5*sqrt(wx)/&numlines.;
		_relx = &boxscale.*4.0*sqrt(wx)*&ppline./&xpix.;
		%system(C,C,2);		* Relative % of proc area. ;
		%bar(-_relx, -_rely, 2*_relx, 2*_rely, green, 0, S);
	end;
	%system(2,2,2);		* Data values. ;
	%line(lowerci, ord-&ticksz., lowerci, ord+&ticksz., black, 1, 0.2);
	%line(upperci, ord-&ticksz., upperci, ord+&ticksz., black, 1, 0.2);

run;

* Define the TEXT columns. ;
data cols(drop=ord col: rectype);
	set plotbase(keep=ord col: rectype);
	array vars[*] col:;
	%dclanno;
	%system(1, 2, 1);		* X abs %, Y data;
	do _i=1 to dim(vars);
		if not missing(vars[_i]) then do;
			if _i lt dim(vars) then xpos=&stcolw.+_i*&dtcolw.;
			                   else xpos=&pltper.-2;
			if rectype eq 'DAT' then do;
		  	%label(xpos, ord, vars[_i], black, 0, 0, &textht., &tfont. , 4);
			end;
			else do;
		  	%label(xpos, ord, vars[_i], black, 0, 0, &textht., &bfont. , 4);
			end;
		end;
	end;
run;

* Column headers. ;
data heads(drop=ord lab: rectype);
	set plotbase(keep=ord lab: rectype);
	if _N_ eq 1;
	array vars[*] lab:;
	%dclanno;
	%system(1, 2, 1);		* X abs %, Y data;
	do _i=1 to dim(vars);
		if not missing(vars[_i]) then do;
			pos = &stcolw.+_i*&dtcolw.;
			if _i eq dim(vars) then pos = &pltper.;
	  	%label(pos, 0.5, vars[_i], black, 0, 0, &textht., &bfont. , 4);
		end;
	end;
 	%label(&stcolw.+2*&dtcolw., 1.5, "&cmpname.", BLACK, 0, 0, &textht., &bfont. , 4);
	%label(&stcolw.+5*&dtcolw., 1.5, "&trtname.", BLACK, 0, 0, &textht., &bfont. , 4);
run;

* Define the OVERALL red diamonds. ; 
data diamond(drop=ord es lowerci upperci rectype);
	set plotbase(keep=ord es lowerci upperci rectype);
	where rectype eq 'SBG';
	%dclanno;
	when = 'A';
	%system(2, 2, 5);		* X data, Y data;
	%poly(lowerci, ord, red, s, 1);
	%polycont(es, ord+0.5*&diaht., red);
	%polycont(upperci, ord, red);
	%polycont(es, ord-0.5*&diaht., red);
	%polycont(lowerci, ord, red);
run;

* Define the Study LABEL column. ;
data label(drop=_i drugord rectype ord study drug sgline:);
	set plotbase(keep=drugord rectype ord study drug sgline: );
	by drugord;
	%dclanno;
	%system(1, 2, 5);		* X abs %, Y data;

	if first.drugord then do;
		%label(1, ord+1, drug, BLACK, 0, 0, &textht.,&bfont. , 6);
	end;
	array lab[*] study sgline:;

	if rectype in ('DAT') then do;
		%label(1, ord, study, black, 0, 0, &textht.,  , 6);
	end;
	else if rectype in ('SBG') then do;
		%label(1, ord, lab[1], BLACK, 0, 0, &textht.,&bfont. , 6);
		do _i=2 to dim(lab);
			if not missing(lab[_i]) then do;
				%label(1, ord+1-_i, lab[_i], BLACK, 0, 0, &textht.,&ifont. , 6);
			end;
		end;
	end;
	else do;
		put 'WARN' 'ING: Unknown RECTYLE ' rectype=;
	end;
run;

* Define the vertical reference lines. ;
data reflines;
	%dclanno;
	when = 'B';
	hsys = '5';
	%system(2, 2, 1);		* X data, y data ;

	* Reference line at the overall diff. ;
	%line(&oves., -&numlines., &oves, 0, green, 2, 0.12);

	* Reference line at zero difference. ;
	%line(0, -&numlines., 0, 0, black, 1, 0.06);

	* Line separating the column headers from the data. ;
	%system(1, 1, 1);
	%line(&pltper.-4, 0, 100, 0, black, 1, 0.2);

	* X axis line at the foot of the plot. ;
	%system(1, 2, 1);
	%line(0, -0.5, 100, -0.5, black, 1, 0.2);
run;

* Favours labels. ;
data favlab;
	%dclanno;
	%system(2, 2, 1);		* X data, y data ;
	%label(-1, -&numlines., "Favours &cmpname.", black, 0, 0, &textht.,&tfont. , 4);
	%label(+1, -&numlines., "Favours &trtname.", black, 0, 0, &textht.,&tfont. , 6);
run;

* All of the annotation together in one data set. ;
data anno;
	set 
		label
		heads
		reflines
		box
		cols
		diamond
		favlab
	;
run;

* Output name ;
filename output "&opfile.";

* Define the size of the plot and the device to use (PNG) ;
goptions reset=all 
         dev=png300 gsfname=output gsfmode=replace
         hsize=&xwid. in vsize=&ywid. in;

* Plot options. ;
symbol1 i=j;
axis1 offset=(&pltper.pct,0) color=black label=none; 
axis2 major=none minor=none label=none color=white value=none order=(-&numlines. to 2);

* The PLOT is only used as a base for the annotations and 
* ensure that the axes are defined. ;
proc gplot data=plotdata;
				plot ord*lval /
							skipmiss
							haxis=axis1
							vaxis=axis2
							annotate=anno
;
run;
quit;
