* Graphics for meta analysis - Figure 2.1
* Standard funnel plot of the SSRI data.
*
* Jeff Tomlinson, November 2011
;

* Get the study information from the SSRI and SSRI2 spreadsheets.  
* We cannot merge using STUDY because study 62 is duplicated.  Assume 
* the sudies in both spreadsheets are in the same order.  Error is 
* there is a discrepancy. ;
libname xla excel './ssri.xls' access=readonly;
libname xlb excel './ssri2.xls' access=readonly;

data ssri;
	merge xla.'GenStat Data$'n
	      xlb.'Sheet1$'n(rename=(study=study2))
	;
	if study ne study2 then put 'ERR' 'OR: Spreadsheets not sorted the same.';
run;


* Plot/axis labelling ;
%let title=;
%let xlabel=Difference between SSRI and Placebo;
%let ylabel=Standard error of difference;
%let maxyax=5;        
%let minyax=0;
%let maxxax=12.5;
%let minxax=-7.5;
%let xaxby=2.5;

* Data set fields in DSIN. ;
%let dsin=ssri;       * Input dataset name. ;
%let drug=drug;       * Variable containing the drug used ;
%let study=study;     * Variable containing the name of the study. ; 
%let numr=n1;         * Number of subjects in active treatment arm, ;
%let numc=n2;         * Number of subjects in comparator arm. ;
%let sed=sed;         * SE of the difference for the study. ;

* Look and style of the plot. ;
%let odstemplate=STYLES.STATISTICAL;   * ODS style to use. ;
%let ftrans=0.8;      * transparency for the contour blocks. ;
%let fcolor=green;    * Colour of the contour blocks. ;
%let scolor=red;			* Colour of study symbols. ;
%let pltht=4in;       * Height of plot (with units) ;
%let pltwd=6in;       * Width of plot (with units) ;

* Other parameters ;
%let imagename=Fig2_1;  * Output PNG filename.  Will have ".png" appended to it. ;

* Dataset structure :
* DRUG  - Drug code/name (Character)
* STUDY - Study name/reference (Character)
* SED - Stnadard error of the difference
* NUMR  - Number on treatment
* NUMC  - Number on compatator
;

* Macro vars populated from the data. ;
%global ovs;  * OVS;
%global n;    * Number of studies.;
%global totn; * Total number of patients in all studies.;
%global mean; * Mean difference.;
%global maxn; * Number of patients in largest study.;
%global minn; * Number of patients in smallest study.;

* Study data to plot - DIFF by 1/SED = OOXCK2 ;
data plotx(keep=drug study diff sed);
  set &dsin. end=atend;
  attrib
    bign  label='Number in  study - Active + Comparator'
    sd    label='SD of the difference'
    topx  label='Sum (n-1)*SD^2 for OVS calc'
    botx  label='Sum (n-1) for OVS calc'
    topm  label='Sum diff*n for mean diff calc'
    totbign label='Total number of subjects'
    minbign label='Smallest number of patients in a study'
    maxbign label='Largest number of patients in a study'
    mindiff label='Minimum difference'
    maxdiff label='Maximum difference'
  ;
  retain minbign maxbign mindiff maxdiff;

  bign = &numr.+&numc.;
  sd = sqrt(bign)*&sed.;
  topx + (bign-1)*sd*sd;
  botx + (bign-1);
  topm + diff*bign;
  totbign + bign;
  minbign = min(minbign, bign);
  maxbign = max(maxbign, bign);
  mindiff = min(mindiff, diff);
  maxdiff = max(maxdiff, diff);

  if atend then do;
    call symputx('OVS',  sqrt(topx/botx));
    call symputx('N',    _n_);
    call symputx('TOTN', totbign);
    call symputx('MEAN', topm/totbign);
    call symputx('MINN', minbign);
    call symputx('MAXN', maxbign);
    call symputx('MINDIFF', round(mindiff-1,2));
    call symputx('MAXDIFF', round(maxdiff+1,2));
    end;
run;


%put ;
%put Number of studies (N) = &n.;
%put Total number of patients in all studies (TOTN) = &totn.;
%put Number of patients in the smallest study (MINN) = &minn.;
%put Number of patients in the largest study (MAXN) = &maxn.;
%put ;
%put Minimum difference (MINDIFF) = &mindiff;
%put Maximum difference (MAXDIFF) = &maxdiff;
%put ;
%put OVS = &ovs.;
%put ;

data conx; *(keep=y u lim:);
  se = &maxyax.;
  ygl = se;
  xgl = &mean.+probit(0.025)*se;
  output;
  ygl = 0;
  xgl = &mean.;;
  output;
  ygl = se;
  xgl = &mean.+probit(0.975)*se;
  output;
run;

* Create a format to give the absolute value for the Y axis label (integers) ;
proc format;
   picture absv (round)
           low - 0  =  '009'
;
run;

* Dataset to plot - Scatter of study 1/SED vs diff and contours. ;
data plot;
  set plotx conx;
  format sed ygl absv.;

  if n(&sed.) then sed = -&sed.;
  if n(ygl) then ygl = -ygl;
run;

* Template for the plot, based on standard statistical template
* Allows for the colours etc to be changed. ;
proc template;   
   define style RSTATISTICAL;
      parent=&odstemplate.;
   end;
run;

* Setup the output PNG filename and size.
* PNG output at 200dpi using the template defined above.;
ods listing image_dpi=200 gpath="./" style=rstatistical;
ods graphics / reset
              imagename="&imagename."
              imagefmt=png
              width=&pltwd.
              height=&pltht.
;

title "&title. ";
footnote;

proc sgplot data=plot noautolegend;
  xaxis values=(&minxax. to &maxxax. by &xaxby.) 
        offsetmin=0.05 
        label="&xlabel. "
  ;
  yaxis min=-&maxyax. max=-&minyax.
        label="&ylabel. "
  ;
  refline &mean. /axis=x lineattrs=(color=&fcolor.);
  series y=ygl x=xgl / lineattrs=(color=&fcolor.)
  ;

  scatter y=sed x=diff / markerattrs=(size=7px color=&scolor. symbol="circlefilled"  ) 
                         name='scat' 
                         nomissinggroup
  ;
run;

