* Graphics for meta analysis - Figure 2.2
* Enhanced funnel plot of the SSRI data -
*    contours correspond to significance levels 0.01, 0.05 and 0.1.
*
* Jeff Tomlinson, November 2011
;

* Format to convert drug code to drug name. ;
proc format;
  value $ftrt
    'F' = 'Fluoxetine'
    'N' = 'Nefazadone'
    'P' = 'Paroxetine'
    'V' = 'Ventafaxine'
  ;
run; 

* Get the study information from the SSRI and SSRI2 spreadsheets.  
* We cannot merge using STUDY because study 62 is duplicated.  Assume 
* the sudies in both spreadsheets are in the same order.  Error is 
* there is a discrepancy. ;
libname xla excel './ssri.xls' access=readonly;
libname xlb excel './ssri2.xls' access=readonly;

data ssri;
	merge xla.'GenStat Data$'n
	      xlb.'Sheet1$'n(rename=(study=study2))
	;
  format drug $ftrt.;
	if study ne study2 then put 'ERR' 'OR: Spreadsheets not sorted the same.';
run;

* Plot/axis labelling ;
%let title=;
%let xlabel=Difference between SSRI and Placebo;
%let ylabel=1/Standard error of difference;
%let maxyax=1.2;
%let minyax=0.2;

* Data set fields in DSIN. ;
%let dsin=ssri;       * Input dataset name. ;
%let drug=drug;       * Variable containing the drug used ;
%let study=study;     * Variable containing the name of the study. ; 
%let numr=n1;         * Number of subjects in active treatment arm, ;
%let numc=n2;         * Number of subjects in comparator arm. ;
%let sed=sed;         * SE of the difference for the study. ;

* Look and style of the plot. ;
%let odstemplate=STYLES.STATISTICAL;   * ODS style to use. ;
%let ftrans=0.8;      * transparency for the contour blocks. ;
%let fcolor=green;    * Colour of the contour blocks. ;
%let pltht=4in;       * Height of plot (with units) ;
%let pltwd=6in;       * Width of plot (with units) ;

* Other parameters ;
%let imagename=Fig2_2;  * Output PNG filename.  Will have ".png" appended to it. ;

* Get the input data into drug/study order. ;
proc sort data=&dsin. out=locdata;
  by &drug. &study.;
run;


* Dataset structure :
* DRUG  - Drug code/name (Character)
* STUDY - Study name/reference (Character)
* SED   - Standard error of the difference
* NUMR  - Number on treatment
* NUMC  - Number on comparator
;

* Macro vars populated from the data. ;
%global ovs;  * OVS;
%global n;    * Number of studies.;
%global totn; * Total number of patients in all studies.;
%global mean; * Mean difference.;
%global maxn; * Number of patients in largest study.;
%global minn; * Number of patients in smallest study.;

* Study data to plot - DIFF by 1/SED = OOXCK2 ;
data plotx(keep=drug study diff ooxck2);
  set locdata end=atend;
  attrib
    ooxck2  label='Inverse of SED (to plot)'
    bign  label='Number in  study - Active + Comparator'
    sd    label='SD of the difference'
    topx  label='Sum (n-1)*SD^2 for OVS calc'
    botx  label='Sum (n-1) for OVS calc'
    topm  label='Sum diff*n for mean diff calc'
    totbign label='Total number of subjects'
    minbign label='Smallest number of patients in a study'
    maxbign label='Largest number of patients in a study'
    mindiff label='Minimum difference'
    maxdiff label='Maximum difference'
  ;
    retain minbign maxbign mindiff maxdiff;

  ooxck2 = 1/&sed.;
  bign = &numr.+&numc.;
  sd = sqrt(bign)*&sed.;
  topx + (bign-1)*sd*sd;
  botx + (bign-1);
  topm + diff*bign;
  totbign + bign;
  minbign = min(minbign, bign);
  maxbign = max(maxbign, bign);
  mindiff = min(mindiff, diff);
  maxdiff = max(maxdiff, diff);

  if atend then do;
    call symputx('OVS',  sqrt(topx/botx));
    call symputx('N',    _n_);
    call symputx('TOTN', totbign);
    call symputx('MEAN', topm/totbign);
    call symputx('MINN', minbign);
    call symputx('MAXN', maxbign);
    call symputx('MINDIFF', round(mindiff-1,2));
    call symputx('MAXDIFF', round(maxdiff+1,2));
    end;
run;


%put ;
%put Number of studies (N) = &n.;
%put Total number of patients in all studies (TOTN) = &totn.;
%put Number of patients in the smallest study (MINN) = &minn.;
%put Number of patients in the largest study (MAXN) = &maxn.;
%put ;
%put Minimum difference (MINDIFF) = &mindiff;
%put Maximum difference (MAXDIFF) = &maxdiff;
%put ;
%put OVS = &ovs.;
%put ;

* Contours at 1%, 5%, 10%, 90%, 95% and 99%;
* 2.576 = 0.005
* 1.960 = 0,025
* 1.645 = 0.050
* PROBIT(P)
;

* Construct the contouts
* Points until 10 times the number in the largest study, but limit to
* the max Y axis value.
*
* To give the contouts at 90%, 95% and 99%, we will need to plot the
* values at 0.5% to 99.5% for 99% between
*           2.5% to 97.5% for 95%
*           5.0% to 95.0% for 90% 
;

data conx(keep=y u lim:);
  array p[6] _temporary_ (0.005 0.025 0.050 0.950 0.975 0.995);
  array lim[6]            lim99l lim95l lim90l lim90u lim95u lim99u;

  u = &maxyax.+0.1;
  do n=2 to 10*&maxn.;
    y = sqrt(n)/&ovs.;
    if y gt &maxyax. then leave;
    do _i=1 to dim(lim);
      lim[_i] = &mean.+probit(p[_i])*&ovs. /sqrt(n); 
    end;
    output;
  end;
run;

* Dataset to plot - Scatter of study 1/SEC vs diff and contours. ;
data plot;
  set plotx conx;
run;

* Template for the plot, based on standard statistical template, but 
* using dofferent symbols and colours. ;
proc template;   
   define style RSTATISTICAL;
      parent=&odstemplate.;

      style GraphData1 from GraphData1 / markersymbol="circlefilled"   LineStyle=1 contrastcolor=red;
      style GraphData2 from GraphData2 / markersymbol="squarefilled"   LineStyle=1 contrastcolor=blue;
      style GraphData3 from GraphData3 / markersymbol="diamondfilled"  LineStyle=1 contrastcolor=royalblue;
      style GraphData4 from GraphData4 / markersymbol="trianglefilled" LineStyle=1 contrastcolor=purple;

   end;
run;


* Setup the output PNG filename and size.
* PNG output at 200dpi using the template defined above.;
ods listing image_dpi=200 gpath="./" style=rstatistical;
ods graphics / reset
              imagename="&imagename."
              imagefmt=png
              width=&pltwd.
              height=&pltht.
;

title "&title. ";
footnote;

proc sgplot data=plot;
  xaxis values=(&mindiff. to &maxdiff. by 2) 
          offsetmin=0.05 
          label="&xlabel."
  ;
  yaxis min=&minyax. max=&maxyax. 
      label="ylabel."
  ;
  refline &mean. /axis=x;

  band x=lim99l lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim95l lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim90l lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim90u lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim95u lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim99u lower=y upper=u /transparency=&ftrans. fillattrs=(color=&fcolor.);

  scatter y=ooxck2 x=diff / group=&drug. 
                            markerattrs=(size=8px) 
                            name='scat' 
                            nomissinggroup
  ;

  keylegend 'scat' /across=1 
                    noborder 
                    location=inside 
                    position=topleft
  ;
run;
