* Graphics for meta analysis - Figure 2.3
* Funnel plot of Clopidogrel data 
*
* Jeff Tomlinson, January 2012
;

* Get the Clopidogrel data directly from the spreadsheet. 
* First define a LIBNAME for the Excel spreadsheet. ;
libname xl excel './excel/clopidogrel.xls' access=readonly;

* Plot/axis labelling ;
%let maxyax=0.5;      * Maximum Y axis (SE of Log risk ratio) ;
%let minyax=0;        * Minimum Y axis should be zero. ;
%let yaxby=0.1;       * BY value for the Y (SE) axis. ;
%let xaxhigh=10;      * High value for the X (Risk Ratio) axis. ;
%let xaxlow=0.1;      * Low value for the X axis (inverse of the high value for a
                      * symmetrical plot. ;
%let title=;          * Title. ;
%let footnote=;       * Footnote ;
%let xlabel=Risk ratio of *2-*8 to *1 or *17;  * X axis label;
%let ylabel=Standard error of log risk ratio;  * Y axis label ;

* Data set fields in DSIN. ;
%let dsin=xl.'GenStat Data$'n;  * Input dataset name. (Sheet of the Excel workbook).;
%let study=study;     * Variable containing the name of the study. ; 
%let numr=n1;         * Number of subjects in active treatment arm, ;
%let nevr=r1;         * Number of subjects with event in active treatment ;
%let numc=n2;         * Number of subjects in comparator arm. ;
%let nevc=r2;         * Number of subjects with event in comparator arm. ;
%let rr=rr;           * Risk ratio ;
%let ci95hi=hi95;     * High 95% CI for RR ;

* Look and style of the plot. ;
%let odstemplate=STYLES.STATISTICAL;   * ODS style to use. ;
%let ftrans=0.8;      * transparency for the contour blocks. ;
%let fcolor=green;    * Colour of the contour blocks. ;
%let scolor=red;      * Colour of study symbols. ;
%let pltht=4in;       * Height of plot (with units) ;
%let pltwd=6in;       * Width of plot (with units) ;

* Other parameters ;
%let topn=0;          * label the largest TOPN studies plotted ;
%let lnse=0;          * Label within contour 0=No labels, 1=Outside 95%,
                      * 2=Outside 99%, 3=Outside 99.5%;
%let imagename=Fig2_3;  * Output PNG filename.  Will have ".png" appended to it. ;

* Construct the plot data. 
* Require - Log of RR and the SE or Log RR to plot.
* STUDY is required if any points are to be labelled.
* TOTAL can be used when the largest studies are to be labelled.
*
* With this data, RR and the 95% CI are given.  From this the SE of the
* log RR can be calculated. 
; 
data fundat(keep=&study. &ci95hi. &rr. loghi logrr lse total);
  set &dsin.;
  if n(&ci95hi., &rr.) ne 2 then do;
    put 'Not enough data to plot ' &study.= &rr.= &ci95hi.=;
    delete;
  end;
  loghi = log10(&ci95hi.);
  logrr = log10(&rr.);
  lse = (loghi-logrr)/1.96;
  total = &numr.+&numc.;
run; 

* Label studies outside the 95%, 99% or 99.5% CI contours.  
* Also label the largest TOPN studies. ;
proc sort data=fundat;
   by descending total;
run;

data funlab1;
  set fundat;
  by descending total;

  attrib
    datlab  length=$20 label='Label for data points of interest'
  ;
  array phigh[3] _temporary_ (0.950 0.975 0.995);

  if _N_ le &topn. then do;
    datlab = &study.;
  end;

  if &lnse. gt 0 then do;
    nse = abs(logrr/(lse*probit(phigh[&lnse.])));
    if nse gt 1 then do;
      datlab = study;
    end;
  end;
run;

* Construct the CI "contours" for the plot.
* These consist of 3 levels at 90%, 95% and 99% Each of these levels will
* be one variable.  
* Each "contour" has 3 points 
*    1) Max SE to plot for low OR
*    2) 0 SE, OR=1 
*    3) Max SE to plot for high OR
*
* Calculate the contours to twice the max SE to plot.  This should allow 
* the window that is plotted to contain filled contours.  Otherwise when
* the contours met the X axis, they would finish, leaving areas that 
* were not filled in.
;
data conx(keep=yv lim:);
  array plow[3]  _temporary_ (0.005 0.025 0.050);
  array phigh[3] _temporary_ (0.995 0.975 0.950);

  array lim[3] lim1-lim3;
  
  attrib
    yv label='SE(Y) value for contour'
  ;

  yv = 2*&maxyax.;
  do s=1 to dim(plow);
    lim[s] = 10**((probit(plow[s]))*yv);
  end;
  output;

  yv = 0;
  do s=1 to dim(plow);
    lim[s] = 1;
  end;
  output;

  yv = 2*&maxyax.;
  do s=1 to dim(phigh);
    lim[s] = 10**((probit(phigh[s]))*yv);
  end;
  output;
run;

* Create a format to give the absolute value for the Y axis label (1 dp) ;
proc format;
   picture absv (round)
           low - 0  =  '009.9'
;
run;

* Dataset to plot 
* 1. Add in the contours. 
* 2. reverse the Y axis (SE) values.  SAS 9.3 allows a reversed axis, so this step will not be required. ;
data plot;
  set funlab1(in=infnl)
      conx(in=incon);
  format lse absv.;
  if n(lse) then lse=-lse;
  if n(yv) then yv=-yv;
run;

* Setup the output PNG filename and size.
* PNG output at 200dpi using the template given in the ODSTEMPLATE macro variable.;
ods listing image_dpi=200 gpath="./" style=&ODSTEMPLATE.;
ods graphics / reset
              imagename="&imagename."
              imagefmt=png
              width=&pltwd.
              height=&pltht.
;

*Title and footnote.;
title "&title. ";
footnote "&footnote. ";

* Produce the funnel plot using the PLOT data;
proc sgplot data=plot noautolegend;
	
  * Risk ratios plotted on a log axis.  The SG* procedures have more limits 
  * on LOG axes. VALUES cannot be used with a LOG axis, so MIN and MAX are needed
  * and SAS will determine where to put the tick marks.  
  *
  * LOGBASE and LOGSTYLE can be used to effect how SAS displays the ticks.  
  * For this example with axis from 0.1 to 10 ...
  * LOGBASE=2 has labels at 0.125, 0.25, 0.5, 1, 2, 4 etc (evenly spaced)
  * LOGBASE=10 only has labels at 0.1, 1 and 10
  * LOGSTYLE=LINEAR gives labels at 0.1, 1, 2, 3, 4 ...
  * Given the range plotted in this example, LOGBASE=2 gives the most even set
  * of tick marks. 
  ;
  xaxis type=log            
          label="&xlabel."  
          min=&xaxlow.      
          max=&xaxhigh.  
          logbase=2 
          minor
          /*logstyle=linear*/ 
  ;

  * SE (Y) axis - The VALUES statement is used to define the axis value.
  * MIN and MAX would require fewer paremeters and would give ticks at 0.1
  * intervals, but an issue with SAS means that the first tick is labelled 
  * 0.0 (for 0.09 instead of 0.1) ;
  yaxis 
      values=(-&maxyax. to -&minyax. by &yaxby.)
      label="&ylabel." offsetmax=0
  ;

  * Reference line for risk ratio of 1. ;
  refline 1 /axis=x lineattrs=(color=&fcolor.);

  * Bands of 90%, 95% and 99% 
  * The bands are plotted with a non-zero transparency.  This means that 
  * when 2 (or more) bands overaly each other, then the solidity of the 
  * colour is increased. ;
  band x=lim1 lower=yv upper=0 /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim2 lower=yv upper=0 /transparency=&ftrans. fillattrs=(color=&fcolor.);
  band x=lim3 lower=yv upper=0 /transparency=&ftrans. fillattrs=(color=&fcolor.);

  * Display the study Odds ratios against SE 
  * Plot after the bands, with a transparency of 0 to ensure that the 
  * points have the same intensity wherever they are plotted. ;
  scatter y=lse x=rr / markerattrs=(size=7px color=&scolor. symbol="circlefilled"  ) 
                       name='scatter' 
                       datalabel=datlab 
                       nomissinggroup 
                       transparency=0
  ;
run;


