* Graphics for meta analysis - Figure 3.1
* Meta-regression of the SSRI data against mean baseline
*
* Jeff Tomlinson, November 2011
;

libname here './SASdata';

* Construct the data to plot.
* Get the mean baseline from the treatment and comparator baselines.;
* The dataset needs 4 fields.
*   1. Study name (character)
*   2. Baseline value
*   3. Difference in means
*   4. Standard error or difference.
* Calculate these from the SSRI data ;
data ssri(keep=study meanbase diff sed);
  set here.ssri;

  sd1 = change1/d1;
  sd2 = change2/d2;
  sdpl = sqrt(((n1-1)*sd1**2 + (n2-1)*sd2**2)/(n1+n2-2));
  sed = sdpl*sqrt(1/n1+1/n2);

  diff = change1-change2;
  meanbase = (n1*baseline1+n2*baseline2)/(n1+n2);
run;

* Plot/axis labelling. ;
%let title=; *Regression of Baseline on Difference in Means;
%let footnote=;

* ... X axis. ;
%let xlabel=Baseline(Hamilton Scale);
%let xfrom=16;
%let xto=30;
%let xby=2;

* ... Y axis. ;
%let ylabel=Difference in Means (Hamilton Scale);
%let yfrom=-2;
%let yto=10;
%let yby=2;

* Data set and fields. ;
%let dsin=ssri;       * Input dataset name ;
%let study=study;     * Study name (for label if needed) ;
%let xvar=meanbase;   * X (baseline);
%let yvar=diff;       * Y (Final difference);
%let var=sed;         * Standard error of the difference. ;

* Look and feel of the plot ;
%let styles=styles.statistical;   * ODS style to use.  Changes font in PROC GPLOT ;
%let bsize=80;        * Number to scale the bubbles by - 10 is the default. ;

* Other parameters ;
%let topn=1;          * Label the top N studies. ;
%let opfile=Fig3_1.png; * Name of the output PNG file. ;

* Data to plot and analyse.
* Use the inverse of the SED as the weight for the bubbles in the plot
* Numeric TRIALNO for PROC MIXED.
;
proc sort data=&dsin out=bubbledat1;
  by &var. &study.;
run;

data bubbledata(keep=&xvar. &yvar. weight trialno est study stlab);
  set bubbledat1 end=atend;
  attrib
    stlab length=$32 label='Annotate study label'
  ;
  weight = 1/&var.;
  est = 1/weight**2;
  trialno=_n_;
  if trialno le &topn. then stlab = &study.;

  if atend then do;
    call symput('NUMSTD',put(_n_,4.-L));
  end;
run;

* Get the limits for the data - smallest and largest baseline and
* smallest and largest difference in means. ;
proc sql noprint;
  select min(&xvar.), max(&xvar.) into :minbase, :maxbase
    from bubbledata
    where &xvar. is not null
  ;
  select min(&yvar.), max(&yvar.) into :mindiff, :maxdiff
    from bubbledata
    where &yvar. is not null
  ;
quit;

* Calculate the regression line. ;
proc mixed method=ml data=bubbledata;
   class trialno;
   model diff=&xvar. /s cl ;
   repeated /group=trialno;
   parms /parmsdata=bubbledata
          eqcons=1 to &numstd.;
   ods output solutionF=solution;
run;

* Determine the coordinates of the start and end of the regression line. ;
data bubbleline(keep=xfrom yfrom xto yto);
  set solution end=atend;
  * 2 vars - Intercept and DIFF ;
  * Calculate for lower value ;

  do until(atend);
    set solution end=atend;
    effect=upcase(effect);
    if effect eq 'INTERCEPT' then intercept = estimate;
    else if effect eq upcase("&xvar.") then &xvar. = estimate;
  end;
  put 'Formula for the line is y = ' intercept ' + ' &xvar. ' * x';

  xfrom = &minbase.;
  yfrom = intercept+&xvar.*xfrom;
  xto = &maxbase.;
  yto = intercept+&xvar.*xto;
run;

* Use annotate to draw the regression line. ;
data bubbleanno1;
  set bubbleline;
  length function style color $8;
  retain xsys ysys "2" when 'B' style ' ';

  * Zero line ;
  function='MOVE'; x=&xfrom.; y=0; output;
  function='DRAW'; x=&xto.; y=0; color='GRAY'; line=2; size=1; output;

  * Regression line ;
  function='MOVE'; x=xfrom; y=yfrom; output;
  function='DRAW'; x=xto; y=yto; color='RED'; line=1; size=2; output;
run;

* Label the top TOPN studies. ;
data bubbleanno2;
  set bubbledata;
  where stlab is not null;
  length function color $8;
  retain xsys ysys '2' hsys '3';
  function='LABEL'; color='BLACK'; x=&xvar.+weight/2; y=&yvar.; text=stlab; position='3'; size=3; output;
run;

* combine the line and label annotate datasets. ;
data bubbleanno;
  set bubbleanno1 bubbleanno2;
run;

* Set-up the required GOPTIONS to enable the production of the PNG output file.
;
filename gsasname "&opfile.";
goptions reset=all
         device=png
         hsize=9in vsize=6in
         gsfname=gsasname
         gsfmode=replace
;

* For SAS 9, STYLES can be used to control the look of the plots (fonts, background
* default colours) ;
ods listing image_dpi=300 gpath="./" style=&styles.;
ods graphics on /
  reset=all
;

axis1 order=(&xfrom. to &xto. by &xby.)
    label=(h=14pt "&xlabel. ")
    minor=none
    value=(h=14pt)
    offset=(2pct, 2pct)
;
axis2 order=(&yfrom. to &yto. by &yby.)
      label=(h=14pt angle=90 "&ylabel. ")
      minor=none
      value=(h=14pt)
      offset=(2pct, 2pct)
;

title h=14pt "&title. ";
footnote "&footnote. ";

* Produce the bubble plot
* The size of the bubble will be proportional to the number of subjects in the study (W)
* BSIZE is used to scale the bubbles to be a reasonable size (default is 5)
* BSCALE is how the size of the bubble relates to the weight, either AREA or RADIUS
*    for AREA, the area of the bubble is proprtional to the weight.
* ANNOTATE the lines from the ANNO dataset.
;
proc gplot data=bubbledata;
    bubble &yvar.*&xvar.=weight /
        bsize=&bsize.       /* Scale bubbles to a reasonable size on the plot */
        bscale=radius       /* WEIGHT defines the radius of the bubble */
        annotate=bubbleanno /* Add the regression line */
        haxis=axis1         /* X axis */
        vaxis=axis2         /* Y axis */
  ;
run;
quit;

